# plugin init files for Ruby On Rails

require 'amrita2/rails_bridge'

ActionView::Base.register_template_handler "a2html", Amrita2View::Base
ActionView::Base.register_template_handler "a2", Amrita2View::Base

